//+----------------------------------------------------------------------------+
//|                                                              Custom EA.mq4 |
//|                                                              Lukasz Szelag |
//|                                                  http://www.forexabyss.com |
//+----------------------------------------------------------------------------+

#property copyright "Lukasz Szelag"
#property link      "http://www.forexabyss.com"

// Assumptions:
//
// - only market orders are used (no pending orders)
// - maximum number of open orders is 1
// - orders that are not opened by the EA are ignored
// - EA doesn't open new orders if freeze level > 0
//

//+----------------------------------------------------------------------------+
//| Constants.                                                                 |
//+----------------------------------------------------------------------------+

/*
 * Magic number to be assigned to all orders opened by this EA. This unique
 * identifier is used to determine if the given order was opened by this EA.
 */
#define ORDER_MAGIC_NUMBER 12345678

// constants for trade operations

#define TRADE_OP_NONE       0
#define TRADE_OP_OPEN_BUY   1
#define TRADE_OP_OPEN_SELL  2
#define TRADE_OP_CLOSE_BUY  3
#define TRADE_OP_CLOSE_SELL 4

// constants for naming indexes of the openOrders array

#define ORDER_TICKET     0
#define ORDER_TYPE       1
#define ORDER_OPEN_PRICE 2
#define ORDER_LOTS       3
#define ORDER_SL         4
#define ORDER_TP         5
#define ORDER_PROFIT     6
#define ORDER_COMMISSION 7
#define ORDER_SWAP       8

//+----------------------------------------------------------------------------+
//| External variables. These can be changed in the program properties window. |
//+----------------------------------------------------------------------------+

// SL, TP and trailing stop levels in points for new orders. The actual stop
// levels will be the same as requested or the minimum allowed by the broker,
// whichever is less. Trailing stop is disabled if 0.

extern int stopLoss   = 10;
extern int takeProfit = 10;
extern int trailStop  = 10;

// slippage for new orders
extern int slippage = 1;

// TRUE if amount of lots in order is fixed, FALSE if it is calculated as a
// percentage of free margin.

extern bool lotsFixed = TRUE;

// percentage of free margin to calculate amount of lots for new orders
extern double lotMarginPC = 2;
 
// amount of lots for new orders (it will be recalculated if not fixed)
extern double lots = 1;

// periods for short and long MA

extern int shortMAPeriod = 5;
extern int longMAPeriod  = 20;

//+----------------------------------------------------------------------------+
//| Global variables.                                                          |
//+----------------------------------------------------------------------------+

// An array that contains information about specific characteristics of all
// market orders available at the current moment. The array is resized
// dynamically to fit available orders and updated on every tick. Only orders
// opened by this EA are accounted for.
//
double openOrders[][9];

// the number of currently open market orders 
int openOrderCount = 0;

// Actual SL, TP and trailing stop levels in points for new orders. These values
// will be calculated dynamically.

int actualSL = 10;
int actualTP = 20;
int actualTS = 10;

// actual freeze level (calculated dynamically)
int actualFL = 0;

//+----------------------------------------------------------------------------+
//| EA initialization special function.                                        |
//+----------------------------------------------------------------------------+
int init() {
   Print("init");
   return(0);
}

//+----------------------------------------------------------------------------+
//| EA deinitialization special function.                                      |
//+----------------------------------------------------------------------------+
int deinit() {
   Print("deinit");
   return(0);
}

//+----------------------------------------------------------------------------+
//| EA start special function (called on every tick).                          |
//+----------------------------------------------------------------------------+
int start() {
	// data reduction
	if (!isTickValid()) {
		return(0);
	}
	
	// order accounting
	performOrderAccounting();
	
	// calculate trading levels (SL, TP, TS, freeze) for new orders
	calculateTradingLevels();
		
	// trail SL levels of open orders
	updateStopLossLevels();

	// calculate position size for new orders
	calculateOrderLots();

	// calculate a trade signal and act on it
	trade(calculateTradeSignal());
         
	return(0);
}

//+----------------------------------------------------------------------------+
//| Data reduction function. The function returns TRUE if the current tick     |
//| should be processed, otherwise FALSE.                                      |
//|                                                                            |
//| Global variables used:                                                     |
//| 	longMAPeriod                                                           |
//+----------------------------------------------------------------------------+
bool isTickValid() {
   return (Bars >= longMAPeriod);
}

//+----------------------------------------------------------------------------+
//| Order accounting function. The function populates global array openOrders  |
//| with characteristics of all currently open (by this EA) market orders. The |
//| number of market orders is stored in global variable openOrderCount.       |
//|                                                                            |
//| Global variables used:                                                     |
//| 	openOrders                                                             |
//| 	openOrderCount                                                         |
//+----------------------------------------------------------------------------+
void performOrderAccounting() {
	int ordersTotal = OrdersTotal();
	
	// resize the order array to fit up to the total number of orders
	ArrayResize(openOrders, MathMax(ordersTotal, 1));
	
	openOrderCount = 0;
	
	for (int i = 0; i < ordersTotal; i++) {
		if (OrderSelect(0, SELECT_BY_POS, MODE_TRADES)) {
			if ((OrderSymbol() == Symbol()) &&
				(OrderMagicNumber() == ORDER_MAGIC_NUMBER) &&
				((OrderType() == OP_BUY) || (OrderType() == OP_SELL))) {

				openOrders[i][ORDER_TICKET]     = OrderTicket();
				openOrders[i][ORDER_TYPE]       = OrderType();
				openOrders[i][ORDER_OPEN_PRICE] = OrderOpenPrice();
				openOrders[i][ORDER_LOTS]       = OrderLots();
				openOrders[i][ORDER_SL]         = OrderStopLoss();
				openOrders[i][ORDER_TP]         = OrderTakeProfit();
				openOrders[i][ORDER_PROFIT]     = OrderProfit();
				openOrders[i][ORDER_COMMISSION] = OrderCommission();
				openOrders[i][ORDER_SWAP]       = OrderSwap();

				openOrderCount++;
			}	
		}
		else {
			// add error processing
		}	
	}

	// reduce the order array size if it is too big
	ArrayResize(openOrders, MathMax(openOrderCount, 1));
}

//+----------------------------------------------------------------------------+
//| Function calculates SL, TP and trailing stop and freeze levels for new     |
//| orders and stores the result in global variables actualSL, actualTP,       |
//| actualTS and actualFL.                                                     |
//|                                                                            |
//| Global variables used:                                                     |
//| 	actualSL                                                               |
//| 	actualTP                                                               |
//| 	actualTS                                                               |
//| 	actualFL                                                               |
//+----------------------------------------------------------------------------+
void calculateTradingLevels() {
	int minDist = MarketInfo(Symbol(), MODE_STOPLEVEL);
	
	// use user requested values or the minimum allowed by the broker
	
	actualSL = stopLoss;
	actualTP = takeProfit;
	actualTS = trailStop;
	
	if (actualSL < minDist) {
		actualSL = minDist;
	}
	
	if (actualTP < minDist) {
		actualTP = minDist;
	}

	// TS is disabled if 0
	if ((actualTS > 0) && (actualTS < minDist)) {
		actualTS = minDist;
	}

	actualFL = MarketInfo(Symbol(), MODE_FREEZELEVEL);
}

//+----------------------------------------------------------------------------+
//| Function updates SL levels of open orders according to trailing stop.      |
//| Prior to calling this function trailing stop level should be updated with  |
//| calculateStopLevels().                                                     |
//|                                                                            |
//| Global variables used:                                                     |
//| 	actualTS                                                               |
//+----------------------------------------------------------------------------+
void updateStopLossLevels() {
	// trailing stop is disabled	
	if (actualTS <= 0) {
		return;
	}
		
	for (int i = 0; i < openOrderCount; i++) {
		int ticket        = openOrders[i][ORDER_TICKET];
		int type          = openOrders[i][ORDER_TYPE];
		double openPrice  = openOrders[i][ORDER_OPEN_PRICE];
		double lots       = openOrders[i][ORDER_LOTS];
		double stopLoss   = openOrders[i][ORDER_SL];
		double takeProfit = openOrders[i][ORDER_TP];
		double profit     = openOrders[i][ORDER_PROFIT];
		double commission = openOrders[i][ORDER_COMMISSION];
		double swap       = openOrders[i][ORDER_SWAP];
					
		double minTS, minSL;

		bool modifyOrder = FALSE;
				
		switch(type) {
			case OP_BUY:
				// adjust trailing stop for freeze level
				minTS = MathMax(actualTS, actualFL);
							
				// if the profit exceeds the trailing stop
				if (NormalizeDouble(Bid - openPrice, Digits) >
					NormalizeDouble(minTS * Point, Digits)) {

					// minimum SL level
					minSL = NormalizeDouble(Bid - minTS * Point, Digits);
				
					// trail order SL level if below minimum
					if (NormalizeDouble(stopLoss, Digits) < minSL) {
						stopLoss    = minSL;
						modifyOrder = TRUE;
					}
				}
					
				break;
					
			case OP_SELL:
				// adjust trailing stop for freeze level
				minTS = MathMax(actualTS, actualFL);

				// if the profit exceeds the trailing stop
				if (NormalizeDouble(openPrice - Ask, Digits) >
					NormalizeDouble(minTS * Point, Digits)) {
			
					// minimum SL level
					minSL = NormalizeDouble(Ask + minTS * Point, Digits);
				
					// trail order SL level if above minimum
					if (NormalizeDouble(stopLoss, Digits) > minSL) {
						stopLoss = minSL;
						modifyOrder = TRUE;
					}	
				}
				
				break;
					
			default:
				continue;
		}
			
		if (modifyOrder) {
			OrderModify(ticket, openPrice,
				NormalizeDouble(stopLoss, Digits),
				NormalizeDouble(takeProfit, Digits), 0);
		}
	}
}
								
//+----------------------------------------------------------------------------+
//| Function calculates the amount of lots for new orders or 0 if there is not |
//| enough free margin to open a position. Amount of lots is either fixed or   |
//| calculated as a percentage of free margin. In both cases the final amount  |
//| of lots is adjusted for conditions (restrictions) set by the broker. The   |
//| result is stored in a global variable lots.                                |
//|                                                                            |
//| Global variables used:                                                     |
//| 	lotsFixed                                                              |
//| 	lotMarginPC                                                            |
//| 	lots                                                                   |
//+----------------------------------------------------------------------------+
void calculateOrderLots() {
	double minLot     = MarketInfo(Symbol(), MODE_MINLOT);
	double maxLot     = MarketInfo(Symbol(), MODE_MAXLOT);
	double lotStep    = MarketInfo(Symbol(), MODE_LOTSTEP);
	double lotPrice   = MarketInfo(Symbol(), MODE_MARGINREQUIRED);
	double freeMargin = AccountFreeMargin();

	if (!lotsFixed) {
		lots = (freeMargin * lotMarginPC / 100) / lotPrice;
		lots = MathFloor(lots / lotStep) * lotStep;
	}
	
	if (lots < minLot) {
		lots = minLot;
	}	
	else if (lots > maxLot) {
		lots = maxLot;
	}
	
	if (lots * lotPrice > freeMargin) {
		lots = 0;
	}
}

//+----------------------------------------------------------------------------+
//| A function that defines trading criteria and calculates a trade signal. It |
//| returns one of the predefined trade operation types or TRADE_OP_NONE       |
//| indicating no trade signal.                                                |
//|                                                                            |
//| Global variables used:                                                     |
//| 	shortMAPeriod                                                          |
//| 	longMAPeriod                                                           |
//| 	openOrderCount                                                         |
//+----------------------------------------------------------------------------+
int calculateTradeSignal() {
	// the zero element is not used
	double shortMA[3], longMA[3];

	// trading criteria are based on completed bars
	if (!isNewBar()) {
		return(TRADE_OP_NONE);
	}
	
	// based on the last completed bar
      
	shortMA[1] = iMA(NULL, 0, shortMAPeriod, 0, MODE_EMA, PRICE_CLOSE, 1);
	longMA[1]  = iMA(NULL, 0, longMAPeriod, 0, MODE_EMA, PRICE_CLOSE, 1);

	// based on the last but one completed bar
      
	shortMA[2] = iMA(NULL, 0, shortMAPeriod, 0, MODE_EMA, PRICE_CLOSE, 2);
	longMA[2]  = iMA(NULL, 0, longMAPeriod, 0, MODE_EMA, PRICE_CLOSE, 2);

	// the trading strategy permits no more than 1 open orders

	if (openOrderCount == 1) {
		// criteria for closing orders

		// closing price crosses below short MA
		if (crossBelow(Close[2], Close[1], shortMA[2], shortMA[1])) {
			return(TRADE_OP_CLOSE_BUY);
		}         
    
		// closing price crosses above short MA
		if (crossAbove(Close[2], Close[1], shortMA[2], shortMA[1])) {
			return(TRADE_OP_CLOSE_SELL);
		}
	}
	else if (openOrderCount == 0) {
		// criteria for opening orders

		// short MA crosses above long MA
		if (crossAbove(shortMA[2], shortMA[1], longMA[2], longMA[1])) {
			return(TRADE_OP_OPEN_BUY);
		}

		// short MA crosses below long MA
		if (crossBelow(shortMA[2], shortMA[1], longMA[2], longMA[1])) {
			return(TRADE_OP_OPEN_SELL);
		}
	}
   
	return(TRADE_OP_NONE);
}

//+----------------------------------------------------------------------------+
//| A function that realizes the trading strategy given a trade operation      |
//| type.                                                                      |
//+----------------------------------------------------------------------------+
int trade(int tradeOp) {
	// don't trade if freeze level is > 0
	if (actualFL > 0) {
		return(0);
	}	
	
	int ticket  = openOrders[0][ORDER_TICKET];
	double orderLots = openOrders[0][ORDER_LOTS];
	
	switch(tradeOp) {
		case TRADE_OP_OPEN_BUY:
			// if there is enough money
			if (lots > 0) {
				OrderSend(Symbol(), OP_BUY, lots, Ask, slippage,
					NormalizeDouble(Bid - (actualSL * Point), Digits),
					NormalizeDouble(Bid + (actualTP * Point), Digits),
					"Comment", ORDER_MAGIC_NUMBER, 0, Green);

				break;
			}	
			
		case TRADE_OP_OPEN_SELL:
			// if there is enough money
			if (lots > 0) {
				OrderSend(Symbol(), OP_SELL, lots, Bid, slippage,
					NormalizeDouble(Ask + (actualSL * Point), Digits),
					NormalizeDouble(Ask - (actualTP * Point), Digits),
					"Comment", ORDER_MAGIC_NUMBER, 0, Red);
			}
			
			break;

		case TRADE_OP_CLOSE_BUY:
			OrderClose(ticket, orderLots, Bid, slippage, Green);
			break;
			
		case TRADE_OP_CLOSE_SELL:
			OrderClose(ticket, orderLots, Ask, slippage, Red);
			break;

		default:
			break;
	}
}

//+----------------------------------------------------------------------------+
//| Returns TRUE if m data array (identified by m1, m2) crosses above n data   |
//| array (identified by n1, n2), otherwise FALSE.                             |
//+----------------------------------------------------------------------------+
bool crossAbove(double m1, double m2, double n1, double n2) {
	return ((m1 < n1) && (m2 > n2));
}
 
//+----------------------------------------------------------------------------+
//| Returns TRUE if m data array (identified by m1, m2) crosses below n data   |
//| array (identified by n1, n2), otherwise FALSE.                             |
//+----------------------------------------------------------------------------+
bool crossBelow(double m1, double m2, double n1, double n2) {
	return ((m1 > n1) && (m2 < n2));
}

//+----------------------------------------------------------------------------+
//| Returns TRUE if a new bar has been formed, otherwise FALSE.                |
//+----------------------------------------------------------------------------+
bool isNewBar() {
	static datetime lastTime = 0;
   
	if (Time[0] != lastTime) {
		lastTime = Time[0];
      
		return(TRUE);
	}
   
	return(FALSE);
}

